@@ -1,22 +1,26 @@
 from future_builtins import map
 import unittest
 import os
-import sys, subprocess
+import sys
+import subprocess
 import heapq
 import time
-import socket, httplib
-import lucene, cherrypy
+import httplib
+import lucene
+import cherrypy
 from lupyne import client, server
 from . import remote
 
+
 def getresponse(error):
     "Test error handling in resources."
     raise error(0)
 
+
 class TestCase(remote.BaseTest):
     ports = 8080, 8081, 8082
     hosts = list(map('localhost:{0:d}'.format, ports))
-    
+
     def testInterface(self):
         "Distributed reading and writing."
         for port in self.ports:
@@ -49,7 +53,7 @@ class TestCase(remote.BaseTest):
         assert len(docs) == len(resources) + 1
         assert len(set(doc['__id__'] for doc in docs)) == 2
         self.stop(self.ports[0])
-        self.assertRaises(socket.error, resources.broadcast, 'GET', '/')
+        self.assertRaises(IOError, resources.broadcast, 'GET', '/')
         assert resources.unicast('GET', '/')()
         del resources[self.hosts[0]]
         assert all(resources.broadcast('GET', '/'))
@@ -63,8 +67,8 @@ class TestCase(remote.BaseTest):
         host = self.hosts[1]
         stream = resources[host].stream('GET', '/')
         resource = next(stream)
-        resource.getresponse = lambda: getresponse(socket.error)
-        self.assertRaises(socket.error, next, stream)
+        resource.getresponse = lambda: getresponse(IOError)
+        self.assertRaises(IOError, next, stream)
         stream = resources[host].stream('GET', '/')
         resource = next(stream)
         resource.getresponse = lambda: getresponse(httplib.BadStatusLine)
@@ -75,7 +79,7 @@ class TestCase(remote.BaseTest):
             client.Pool.resource_class = client.SResource
             self.assertRaises(httplib.ssl.SSLError, client.Pool(self.hosts[-1]).call, 'GET', '/')
         client.Pool.resource_class = client.Resource
-    
+
     def testSharding(self):
         "Sharding of indices across servers."
         for port in self.ports:
@@ -102,12 +106,12 @@ class TestCase(remote.BaseTest):
             zones.update(doc['zone'] for doc in docs)
         assert zones == set('012')
         self.stop(self.ports[0])
-        self.assertRaises(socket.error, shards.broadcast, 0, 'GET', '/')
+        self.assertRaises(IOError, shards.broadcast, 0, 'GET', '/')
         responses = shards.multicast([0, 1, 2], 'GET', '/')
         assert len(responses) == 2 and all(response() for response in responses)
         shards.resources.priority = lambda hosts: None
         self.assertRaises(ValueError, shards.choice, [[0]])
-    
+
     def testReplication(self):
         "Replication from indexer to searcher."
         directory = os.path.join(self.tempdir, 'backup')
@@ -118,7 +122,6 @@ class TestCase(remote.BaseTest):
         for args in [('-r', self.tempdir), (update, self.tempdir), (update, self.tempdir, self.tempdir)]:
             assert subprocess.call((sys.executable, '-m', 'lupyne.server', sync) + args, stderr=subprocess.PIPE)
         replicas = client.Replicas(self.hosts[:2], limit=1)
-        replicas.discard(None)
         replicas.post('/docs', [{}])
         assert replicas.post('/update') == 1
         resource = client.Resource(self.hosts[2])
@@ -154,7 +157,7 @@ class TestCase(remote.BaseTest):
         root.fields, root.autoupdate = {}, 0.1
         cherrypy.config['log.screen'] = self.config['log.screen']
         cherrypy.engine.state = cherrypy.engine.states.STARTED
-        root.monitor.start() # simulate engine starting
+        root.monitor.start()  # simulate engine starting
         time.sleep(0.2)
         app.root.indexer.add()
         time.sleep(0.2)
