@@ -50,7 +50,7 @@ class TestCase(BaseTest):
     def testInterface(self):
         "Indexer and document interfaces."
         self.assertRaises(TypeError, engine.IndexSearcher)
-        analyzer = lambda reader: analysis.standard.StandardTokenizer(util.Version.values()[-1], reader)
+        analyzer = lambda reader: analysis.standard.StandardTokenizer(util.Version.LUCENE_CURRENT, reader)
         stemmer = engine.Analyzer(analyzer, analysis.en.PorterStemFilter, typeAsPayload)
         for token in stemmer.tokens('hello'):
             assert token.positionIncrement == 1
@@ -62,7 +62,7 @@ class TestCase(BaseTest):
             token.offset, token.positionIncrement = (0, 0), 0
         assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
         assert str(stemmer.parse('hellos', field={'body': 1.0, 'title': 2.0})) == 'body:hello title:hello^2.0'
-        indexer = engine.Indexer(analyzer=stemmer, version=util.Version.LUCENE_46, writeLockTimeout=100L)
+        indexer = engine.Indexer(analyzer=stemmer, version=util.Version.LUCENE_48, writeLockTimeout=100L)
         assert indexer.config.writeLockTimeout == 100
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
@@ -319,11 +319,12 @@ class TestCase(BaseTest):
         assert dict(indexer.positionvector(id, 'text', offsets=True))['persons'] == [(46, 53), (301, 308)]
         analyzer = analysis.core.WhitespaceAnalyzer(util.Version.LUCENE_CURRENT)
         query = indexer.morelikethis(0, analyzer=analyzer)
-        assert str(query) == 'text:united text:states'
+        assert set(str(query).split()) == set(['text:united', 'text:states'])
         hits = indexer.search(query & engine.Query.prefix('article', ''))
         assert len(hits) == 8 and hits[0]['article'] == 'Preamble'
         assert str(indexer.morelikethis(0, 'article', analyzer=analyzer)) == ''
-        assert str(indexer.morelikethis(0, minDocFreq=3, analyzer=analyzer)) == 'text:establish text:united text:states'
+        query = indexer.morelikethis(0, minDocFreq=3, analyzer=analyzer)
+        assert set(str(query).split()) == set(['text:establish', 'text:united', 'text:states'])
         assert str(indexer.morelikethis('jury', 'text', minDocFreq=4, minTermFreq=1, analyzer=analyzer)) == 'text:jury'
         assert str(indexer.morelikethis('jury', 'article', analyzer=analyzer)) == ''
         self.assertRaises(lucene.JavaError, indexer.morelikethis, 'jury')
@@ -622,7 +623,7 @@ class TestCase(BaseTest):
 
     def testNearRealTime(self):
         "Near real-time index updates."
-        indexer = engine.Indexer(version=util.Version.LUCENE_46, nrt=True)
+        indexer = engine.Indexer(version=util.Version.LUCENE_48, nrt=True)
         indexer.add()
         assert indexer.count() == 0 and not indexer.current
         indexer.refresh(filters=True)
@@ -673,13 +674,14 @@ class TestCase(BaseTest):
         parallel.update('charlie', priority='low')
         parallel.commit()
         assert [hit['name'] for hit in indexer.search(filter=filter)] == ['delta']
+        assert indexer.count(filter=indexer.sorter('name').filter('a', 'c'))
 
     def testMulti(self):
         "MultiSearchers."
         indexers = engine.Indexer(self.tempdir), engine.Indexer()
         searcher = engine.MultiSearcher([indexers[0].indexReader, indexers[1].directory])
         self.assertRaises(TypeError, getattr, searcher, 'timestamp')
-        assert engine.MultiSearcher(indexers[:1]).timestamp
+        assert engine.MultiSearcher([indexers[0].directory]).timestamp
         assert [reader.refCount for reader in searcher.indexReaders] == [2, 1]
         assert searcher.reopen() is searcher
         indexers[0].add()
@@ -697,6 +699,7 @@ class TestCase(BaseTest):
         indexer.set('id')
         indexer.set('votes', engine.DocValuesField, type='numeric')
         indexer.set('tag', engine.DocValuesField, type='binary')
+        assert not indexer.comparator('tag')
         indexer.add(id='0', votes=1, tag='low')
         indexer.commit()
         segments = indexer.segments
@@ -706,15 +709,12 @@ class TestCase(BaseTest):
         segments = indexer.segments
         assert list(indexer.comparator('votes', type=int)) == [2]
         assert list(indexer.comparator('tag', type='bytes')) == ['medium']
-        indexer.update('id', '0', votes=3)
+        indexer.update('id', '0', votes=3, tag='high')
         indexer.commit()
         assert indexer.segments == segments
         assert list(indexer.comparator('votes', type=int)) == [3]
-        if lucene.VERSION >= '4.8':
-            indexer.update('id', '0', tag='high')
-            indexer.commit()
-            assert indexer.segments == segments
-            assert list(indexer.comparator('tag', type='bytes')) == ['high']
+        assert list(indexer.comparator('tag', type='bytes')) == ['high']
+        assert indexer.count(filter=indexer.sorter('votes', int).filter(3, 4))
 
 if __name__ == '__main__':
     lucene.initVM()
